from plone.theme.interfaces import IDefaultPloneLayer
from zope.schema.interfaces import IVocabularyFactory


class IFtwUserManagement(IDefaultPloneLayer):
    """ Marker interface for a zope 3 browser layer.
    """


class IUserManagementVocabularyFactory(IVocabularyFactory):
    """ Vocabulary returning groups or users
    """



# coding=UTF-8
from ftw.usermanagement.browser.group.group_add import GroupAdd
from ftw.usermanagement.browser.group.group_delete import GroupDelete
from ftw.usermanagement.browser.group.group_membership import GroupMembership
from ftw.usermanagement.browser.user.user_delete import UserDelete
from ftw.usermanagement.browser.user.user_membership import UserMembership
from ftw.usermanagement.browser.user.user_register import UserRegister
from ftw.usermanagement.interfaces import IFtwUserManagement
from ftw.usermanagement.testing import USERMANAGEMENT_PLONE_LAYER
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase
from zope.interface import directlyProvidedBy, directlyProvides
import re


class MemberIntegrationTests(TestCase):

    layer = USERMANAGEMENT_PLONE_LAYER

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.mtool = getToolByName(self.portal, 'portal_membership')
        self.acl = getToolByName(self.portal, 'acl_users')
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        # Layer for request
        ifaces = [IFtwUserManagement] + list(directlyProvidedBy(self.request))
        directlyProvides(self.request, *ifaces)

        # Set authenticator flag to get a valid request
        value = self.portal.restrictedTraverse(
            '@@authenticator').authenticator()
        auth = re.search('value="(.*)"', value).group(1)
        self.request.form = dict(_authenticator=auth)
        self.request.other['method'] = 'POST'

        # Setup 10 Users
        registration = getToolByName(self.portal, 'portal_registration')
        for i in range(10):
            registration.addMember(
                'user_%s@test.ch' % i,
                '12345',
                properties={
                    'username': 'user_%s' % i,
                    'email': 'user_%s@test.ch' % i,
                    },
                REQUEST=self.layer['request'])

        # Setup 5 Groups
        groups_tool = getToolByName(self.portal, 'portal_groups')
        for i in range(5):
            groups_tool.addGroup('group_%s' % i)

    def test_add_user(self):

        self.request['firstname'] = 'Jämes'
        self.request['lastname'] = 'Bönd'
        self.request['email'] = 'james@bond.ch'

        UserRegister(self.portal, self.request)()

        info = self.mtool.getMemberInfo('james@bond.ch')

        self.assertEquals(info.get('fullname'), 'Bönd Jämes')

    def test_add_group(self):

        self.request['group_id'] = 'test_group'
        self.request['group_title'] = 'Test Gröup'

        GroupAdd(self.portal, self.request)()

        self.assertNotEquals(self.acl.getGroupById('test_group'), None)

    def test_delete_user(self):

        member_1 = self.mtool.getMemberById('user_1@test.ch')
        member_2 = self.mtool.getMemberById('user_2@test.ch')

        self.assertNotEquals(member_1, None)
        self.assertNotEquals(member_2, None)

        self.request['userids'] = ['user_1@test.ch', 'user_2@test.ch']

        UserDelete(self.portal, self.request).delete()

        member_1 = self.mtool.getMemberById('user_1@test.ch')
        member_2 = self.mtool.getMemberById('user_2@test.ch')

        self.assertEquals(member_1, None)
        self.assertEquals(member_2, None)

    def test_delete_group(self):

        group_1 = self.acl.source_groups.getGroupById('group_1')
        group_2 = self.acl.source_groups.getGroupById('group_2')

        self.assertNotEquals(group_1, None)
        self.assertNotEquals(group_2, None)

        self.request['groupids'] = ['group_1', 'group_2']

        GroupDelete(self.portal, self.request).delete()

        group_1 = self.acl.source_groups.getGroupById('group_1')
        group_2 = self.acl.source_groups.getGroupById('group_2')

        self.assertEquals(group_1, None)
        self.assertEquals(group_2, None)

    def test_user_membership(self):

        groups = self.acl.getUserById('user_1@test.ch').getGroupIds()

        self.assertNotIn('group_1', groups)
        self.assertNotIn('group_2', groups)

        self.request['userid'] = 'user_1@test.ch'
        self.request.form.update({'form.submitted': True})
        self.request.form.update({'new_groups': ['group_1', 'group_2']})

        UserMembership(self.portal, self.request)()

        groups = self.acl.getUserById('user_1@test.ch').getGroupIds()

        self.assertIn('group_1', groups)
        self.assertIn('group_2', groups)

    def test_group_membership(self):

        members = self.acl.getGroupById('group_1').getGroupMemberIds()

        self.assertNotIn('user_1@test.ch', members)
        self.assertNotIn('user_2@test.ch', members)

        self.request['group_id'] = 'group_1'
        self.request.form.update({'form.submitted': True})
        self.request.form.update(
            {'new_users': ['user_1@test.ch', 'user_2@test.ch']})

        GroupMembership(self.portal, self.request)()

        members = self.acl.getGroupById('group_1').getGroupMemberIds()

        self.assertIn('user_1@test.ch', members)
        self.assertIn('user_2@test.ch', members)

    def test_user_overview(self):

        view = self.portal.restrictedTraverse(
            '@@tabbedview_view-users_management')()

        # Check for any users in the view
        self.assertIn('user_1@test.ch', view)
        self.assertIn('user_9@test.ch', view)

    def test_group_overview(self):

        view = self.portal.restrictedTraverse(
            '@@tabbedview_view-groups_management')()

        # Check for any groups in the view
        self.assertIn('group_1', view)
        self.assertIn('group_4', view)

    def test_tabbed_management(self):

        view = self.portal.restrictedTraverse('user_management')()

        # Check tabs
        self.assertIn('users_management', view)
        self.assertIn('groups_management', view)



from ftw.testing import MockTestCase
from mocker import ANY
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.group.group_delete import GroupDelete
from Products.statusmessages.interfaces import IStatusMessage
from zope.interface import Interface


class DeleteGroupTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(DeleteGroupTests, self).setUp()

        self.request = {}
        self.context = self.stub()

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

        self.statusmsg = self.mocker.mock(count=False)
        self.message_cache = self.create_dummy()
        self.expect(self.statusmsg(ANY).addStatusMessage(ANY, type=ANY)).call(
            lambda msg, type: setattr(self.message_cache, type, msg))
        self.mock_adapter(self.statusmsg, IStatusMessage, (Interface, ))

    def test_no_id(self):

        self.expect(self.gtool.removeGroups(ANY)).result(True)

        self.replay()

        GroupDelete(self.context, self.request).delete()

    def test_one_id(self):

        self.request['groupids'] = 'groupid'

        self.expect(self.gtool.removeGroups(ANY)).result(True)

        self.replay()

        GroupDelete(self.context, self.request).delete()

        self.assertEquals(self.message_cache.info, u'Group(s) deleted.')

    def test_more_ids(self):

        self.request['groupids'] = ['groupid_1', 'groupid_2']

        self.expect(self.gtool.removeGroups(ANY)).result(True)

        self.replay()

        GroupDelete(self.context, self.request).delete()

        self.assertEquals(self.message_cache.info, u'Group(s) deleted.')

    def test_bad_group_id(self):

        self.request['groupids'] = ['badid', 'groupid_2']

        self.expect(self.gtool.removeGroups(ANY)).throw(KeyError)

        self.replay()

        GroupDelete(self.context, self.request).delete()

        self.assertEquals(
            self.message_cache.error,
            u'There was an error while deleting a group.')



from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.usermanagement import user_management_factory as _
from Products.statusmessages.interfaces import IStatusMessage
from Products.CMFCore.utils import getToolByName


class UserDelete(BrowserView):
    """Get a list of users from REQUEST to delete"""

    template = ViewPageTemplateFile('user_delete.pt')

    def __call__(self):
        return self.template()

    def delete(self):
        """delete users"""
        userids = self.request.get('userids', [])
        mtool = getToolByName(self, 'portal_membership')
        mtool.deleteMembers(
            userids,
            delete_memberareas=1,
            delete_localroles=0,
            REQUEST=self.request,
            )

        IStatusMessage(self.request).addStatusMessage(
            _(u'text_user_deleted'), type="info")



from ftw.testing import MockTestCase
from mocker import ANY
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.group.groups import GroupsSearchResultExecutor
from plone.app.controlpanel.interfaces import IPloneControlPanelView
from zope.interface import Interface


class GroupTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(GroupTests, self).setUp()

        self.request = self.stub_request()

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

    def base_get_groups(self, query, result):

        groups = [
            {'groupid': 'group1', 'title': 'Group1'},
            {'groupid': 'group111', 'title': 'Group111'},
            {'groupid': 'group11', 'title': 'Group11'},
            {'groupid': 'group3', 'title': 'Group3'},
            {'groupid': 'AuthenticatedUsers', 'title': 'Auth'},
            {'groupid': 'group2', 'title': 'Group2'},
        ]

        groupprefs = self.mocker.mock(count=False)
        self.mock_adapter(
            groupprefs,
            IPloneControlPanelView,
            (Interface, Interface),
            name=u'usergroup-groupprefs',
        )
        self.expect(groupprefs(ANY, ANY)).result(groupprefs)

        self.expect(groupprefs.doSearch(searchString=ANY)).result(groups)

        self.replay()

        executor = GroupsSearchResultExecutor(self.context, query)

        self.assertEquals(executor.get_results(), result)

    def test_get_groups_no_filter(self):

        query = {'filter_text': ''}

        result = [
            {'group_id': 'group1', 'group_title': 'Group1'},
            {'group_id': 'group11', 'group_title': 'Group11'},
            {'group_id': 'group111', 'group_title': 'Group111'},
            {'group_id': 'group2', 'group_title': 'Group2'},
            {'group_id': 'group3', 'group_title': 'Group3'},
        ]

        self.base_get_groups(query, result)

    def test_get_groups_with_filter(self):

        query = {'filter_text': 'Group1'}

        result = [
            {'group_id': 'group1', 'group_title': 'Group1'},
            {'group_id': 'group11', 'group_title': 'Group11'},
            {'group_id': 'group111', 'group_title': 'Group111'},
        ]

        self.base_get_groups(query, result)



from ftw.testing import MockTestCase
from mocker import ANY
from ftw.usermanagement.testing import USERMANAGEMENT_ZCML_LAYER
from ftw.usermanagement.browser.user.users import UsersSearchResultExecutor
from ftw.usermanagement.interfaces import IUserManagementVocabularyFactory


class GroupNamesOfUserTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(GroupNamesOfUserTests, self).setUp()

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.context)

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

    def test_no_groups(self):

        self.expect(self.gtool.getGroupsByUserId(ANY)).result([])

        self.replay()

        executor = UsersSearchResultExecutor(self.context, '')
        result = executor.get_group_names_of_user('user')

        self.assertEquals(result, 'No Group')

    def test_unsorted_groups(self):

        groups = [Group('group%s' % i) for i in range(5)]
        groups.reverse()

        self.expect(self.gtool.getGroupsByUserId(ANY)).result(groups)

        self.replay()

        executor = UsersSearchResultExecutor(self.context, '')
        result = executor.get_group_names_of_user('user')

        self.assertEquals(result, 'group0, group1, group2, group3, group4')

    def test_filter_groups(self):

        groups = [Group('group%s' % i) for i in range(5)]
        groups.append(Group('AuthenticatedUsers'))

        self.expect(self.gtool.getGroupsByUserId(ANY)).result(groups)

        self.replay()

        executor = UsersSearchResultExecutor(self.context, '')
        result = executor.get_group_names_of_user('invalid')

        self.assertEquals(result, 'group0, group1, group2, group3, group4')


class UserTests(MockTestCase):

    layer = USERMANAGEMENT_ZCML_LAYER

    def setUp(self):
        super(UserTests, self).setUp()

        self.request = self.stub_request()

        self.context = self.stub()
        self.expect(self.context.REQUEST).result(self.request)

        self.mtool = self.mocker.mock(count=False)
        self.mock_tool(self.mtool, 'portal_membership')

        self.gtool = self.mocker.mock(count=False)
        self.mock_tool(self.gtool, 'portal_groups')

    def base_search_result_executor(self, query, result, users_factory=None):

        # List with users to display
        if not users_factory:
            users_factory = [User('user%s' % str(i/2)) for i in range(5)]

        self.expect(self.gtool.getGroupsByUserId(ANY)).result([])

        # Factory returns the users
        factory = self.mocker.mock(count=False)
        self.expect(factory(ANY)).result(users_factory)
        self.mock_utility(
            factory,
            IUserManagementVocabularyFactory,
            'plone.principalsource.Users',
        )

        self.expect(
            self.mtool.getMemberInfo(ANY)).call(
            lambda x: x != 'broken' and {
                'fullname': '%s Fullname' % x,
                'username': '%s Username' % x} or None)

        self.replay()

        # Executes the query
        executor = UsersSearchResultExecutor(self.context, query)

        self.assertEquals(executor(), result)

    def test_query_no_filter_no_page_no_size(self):
        query = {
            'filter_text': '',
            'batching': True,
            'pagesize': 0,
            'current_page': 0,
        }
        result = [
            {'counter': 1, 'groups': '', 'name': '', 'userid': 'user0'},
            {'counter': 2, 'groups': '', 'name': '', 'userid': 'user0'},
            {'counter': 3, 'groups': '', 'name': '', 'userid': 'user1'},
            {'counter': 4, 'groups': '', 'name': '', 'userid': 'user1'},
            {'counter': 5, 'groups': '', 'name': '', 'userid': 'user2'},
        ]
        self.base_search_result_executor(query, result)

    def test_broke_user(self):

        users_factory = [User('user%s' % str(i)) for i in range(5)]
        users_factory.insert(3, User('broken'))

        query = {
            'filter_text': '',
            'batching': True,
            'pagesize': 50,
            'current_page': 1,
        }
        result = [
            {'counter': 1, 'groups': 'No Group', 'name': 'user0 Fullname',
             'userid': 'user0'},
            {'counter': 2, 'groups': 'No Group', 'name': 'user1 Fullname',
            'userid': 'user1'},
            {'counter': 3, 'groups': 'No Group', 'name': 'user2 Fullname',
             'userid': 'user2'},
            {'counter': 5, 'groups': 'No Group', 'name': 'user3 Fullname',
             'userid': 'user3'},
            {'counter': 6, 'groups': 'No Group', 'name': 'user4 Fullname',
             'userid': 'user4'},
        ]
        self.base_search_result_executor(query, result, users_factory)

    def test_query_no_batching(self):

        query = {
            'filter_text': '',
            'batching': False,
            'pagesize': 50,
            'current_page': 1,
        }
        result = [
            {
                'counter': 1,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 2,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 3,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 4,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 5,
                'groups': 'No Group',
                'name': 'user2 Fullname',
                'userid': 'user2',
            },
        ]
        self.base_search_result_executor(query, result)

    def test_query_batching(self):
        query = {
            'filter_text': '',
            'batching': True,
            'pagesize': 2,
            'current_page': 1,
        }
        result = [
            {
                'counter': 1,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 2,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {'counter': 3, 'groups': '', 'name': '', 'userid': 'user1'},
            {'counter': 4, 'groups': '', 'name': '', 'userid': 'user1'},
            {'counter': 5, 'groups': '', 'name': '', 'userid': 'user2'},
        ]
        self.base_search_result_executor(query, result)

    def test_query_batching_second_page(self):
        query = {
            'filter_text': '',
            'batching': True,
            'pagesize': 2,
            'current_page': 2,
        }
        result = [
            {'counter': 1, 'groups': '', 'name': '', 'userid': 'user0'},
            {'counter': 2, 'groups': '', 'name': '', 'userid': 'user0'},
            {
                'counter': 3,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 4,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {'counter': 5, 'groups': '', 'name': '', 'userid': 'user2'},
        ]
        self.base_search_result_executor(query, result)

    def test_query_filter_and_no_batching(self):
        query = {
            'filter_text': 'user1',
            'batching': False,
            'pagesize': 2,
            'current_page': 1,
        }
        result = [
            {
                'counter': 3,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 4,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
        ]
        self.base_search_result_executor(query, result)

    def test_query_filter_and_batching(self):
        query = {
            'filter_text': 'user',
            'batching': True,
            'pagesize': 2,
            'current_page': 3,
        }
        result = [
            {
                'counter': 1,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 2,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 3,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 4,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 5,
                'groups': 'No Group',
                'name': 'user2 Fullname',
                'userid': 'user2',
            },
        ]
        self.base_search_result_executor(query, result)

    def test_query_bigger_pagesize(self):
        query = {
            'filter_text': '',
            'batching': True,
            'pagesize': 50,
            'current_page': 1,
        }
        result = [
            {
                'counter': 1,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 2,
                'groups': 'No Group',
                'name': 'user0 Fullname',
                'userid': 'user0',
            },
            {
                'counter': 3,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 4,
                'groups': 'No Group',
                'name': 'user1 Fullname',
                'userid': 'user1',
            },
            {
                'counter': 5,
                'groups': 'No Group',
                'name': 'user2 Fullname',
                'userid': 'user2',
            },
        ]
        self.base_search_result_executor(query, result)

    def test_query_bigger_current_page(self):
        query = {
            'filter_text': '',
            'batching': True,
            'pagesize': 50,
            'current_page': 5,
        }
        result = [
            {'counter': 1, 'groups': '', 'name': '', 'userid': 'user0'},
            {'counter': 2, 'groups': '', 'name': '', 'userid': 'user0'},
            {'counter': 3, 'groups': '', 'name': '', 'userid': 'user1'},
            {'counter': 4, 'groups': '', 'name': '', 'userid': 'user1'},
            {'counter': 5, 'groups': '', 'name': '', 'userid': 'user2'},
        ]
        self.base_search_result_executor(query, result)

    def test_query_no_result(self):
        query = {
            'filter_text': 'blabla',
            'batching': True,
            'pagesize': 2,
            'current_page': 1,
        }
        result = []
        self.base_search_result_executor(query, result)


class User(object):
    """ Mockobject representing a user of the principal source
    """

    def __init__(self, id_):
        self.id = id_

    def get(self, attr):
        return self.id


class Group(object):
    """ Mockobject representing a group of the portal_groups utility
    """

    def __init__(self, title):
        self.title = title

    def getGroupTitleOrName(self):
        return self.title

    def getId(self):
        return self.title



from Products.CMFCore.utils import getToolByName
from ftw.usermanagement import user_management_factory as _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from plone.app.controlpanel.usergroups import UsersGroupsControlPanelView


class UserMembership(UsersGroupsControlPanelView):
    """Provides another way to assign groups to a member"""

    template = ViewPageTemplateFile('user_membership.pt')

    def __call__(self):
        """ Check the given request parameters and call the
        correct functions
        """
        form = self.request.form
        user_id = self.request.get('userid', None)
        groups = form.get('new_groups', [])

        if not user_id:
            return 'No user selected'

        if not form.get('form.submitted', False):
            return self.render(user_id)

        return self.replace_groups(user_id, groups)

    def render(self, user_id):
        """ Renders the popup to assign new groups
        """
        users = self.get_display_groups(user_id)
        return self.template(users=users)

    def replace_groups(self, user_id, groups):
        """ Replace the assigned group in the groups-attr
        """
        gtool = getToolByName(self.context, 'portal_groups')
        current_groups = self.get_groups(user_id)

        # Remove unselected groups
        for group in current_groups:
            if group == 'AuthenticatedUsers':
                continue
            if group not in groups:
                gtool.removePrincipalFromGroup(user_id, group, self.request)

        # Add member to groups
        for group in groups:
            if group not in current_groups:
                gtool.addPrincipalToGroup(user_id, group, self.request)

        IStatusMessage(self.request).addStatusMessage(
            _(u'Changes made.'), type="info")

        return True

    def get_groups(self, user_id):
        """Return a sorted list of group ids the user is member of
        """
        gtool = getToolByName(self.context, 'portal_groups')
        mtool = getToolByName(self.context, 'portal_membership')

        member = mtool.getMemberById(user_id)
        groups = gtool.getGroupsForPrincipal(member)

        group_results = [gtool.getGroupById(m) for m in groups]

        group_results.sort(
            key=lambda x: x is not None and x.getGroupTitleOrName().lower())

        group_results = filter(None, group_results)
        group_ids = [group.getId() for group in group_results]
        return group_ids

    def get_display_groups(self, user_id):
        """Returns a list of dicts with name, title and is_member_of"""

        groups = []
        current_groups = self.get_groups(user_id)

        for g in self.membershipSearch(searchUsers=False):
            # Ignore AuthenticatedUsers group
            if g.getId() == 'AuthenticatedUsers':
                continue
            groups.append(dict(
                name=g.getId(),
                title=g.getGroupTitleOrName(),
                is_member_of=g.getId() in current_groups))

        return groups



from ftw.tabbedview.browser.tabbed import TabbedView
from ftw.usermanagement import user_management_factory as _


class ManagementTabbedView(TabbedView):
    """TabbedView for User/Groups management"""

    def __init__(self, context, request):
        super(ManagementTabbedView, self).__init__(context, request)

        if not request.get('disable_border'):
            request.set('disable_border', '1')

        request.set('disable_plone.rightcolumn', True)
        request.set('disable_plone.leftcolumn', True)

    def get_tabs(self):
        """Returns a list of dicts containing the tabs definitions"""

        translate = self.context.translate
        return [{'id':'users_management', 'class':'',
                'description': translate(
                    _(
                        'msg_usersDescription',
                         default='Users-management')), },
                {'id':'groups_management', 'class':'',
                'description': translate(
                    _(
                        'msg_groupsDescription',
                        default='Groups-management')), }, ]
